#!/usr/bin/perl -Tw
################################################################################
package WWW::Yahoo::Briefcase::Object;
# $Id$

=pod

=head1 NAME

WWW::Yahoo::Briefcase::Object - contains information about a file or folder

=head1 SYNOPSIS

  require WWW::Yahoo::Briefcase::Object;

  my $file =
    WWW::Yahoo::Briefcase::Object->new(
				       location => '/',
				       name => 'File',
				       type => '.txt',
				       size => 12345, # bytes
				       time => 1060066800, # 5-Aug-2003
				       id => 'f5f0',
				      );

  my $folder =
    WWW::Yahoo::Briefcase::Object->new(
				       location => '/Parent Folder',
				       name => 'Subfolder',
				       type => 'folder',
				       size => 3, # items
				       time => 1059807600, # 2-Aug-2003
				       id => '38fb',
				      );

=head1 DESCRIPTION

This object represents a specific object in the briefcase (it can be a
file or a folder) and holds various attributes for that object.

=head1 METHODS

=cut

################################################################################
use 5.006;
use strict;
use warnings;
use overload '""' => sub { $_[0]->fullname },
  'cmp' => sub { "$_[0]" cmp "$_[1]" };
require WWW::Yahoo::Briefcase::Exceptions;

our $VERSION = '0.80';
our $AUTOLOAD;
my $autoload =
{ map { $_ => 1 } qw( location name type size time id filename ) };
eval "sub $_" for keys %$autoload; # "forward declare" auto-loaded subs

################################################################################

=pod

=head1 new

  my $object =
    WWW::Yahoo::Briefcase::Object->new(
				       location => $folder,
				       name => $name,
				       type => $extension,
				       size => $size,
				       time => $timestamp,
				       id => $id,
				      );

If the object is a file, the extension, including the period, is used
as the type and the extension is not part of the name.  If the object
is a folder, the type is 'folder' and the size is the number of
objects immediately contained within the folder.  The timestamp is the
last-modified-date, expressed in unix time (seconds since epoch).

=cut

#-------------------------------------------------------------------------------
sub new {
    my $class = shift;

    my $self = bless {}, $class;
    %$self = @_;
    foreach my $name ( keys %$self ) {
	next if $autoload->{$name};
	my $error = "invalid '$name' field in class $class";
	WWW::Yahoo::Briefcase::Exception::Usage->throw( $error );
    }
    if ( exists $self->{filename} ) {
	my $error = "invalid 'filename' field in class $class";
	WWW::Yahoo::Briefcase::Exception::Usage->throw( $error );
    }
    $self;
}

#-------------------------------------------------------------------------------
sub fullname {
    my $self = shift;
    my $name = $self->name;
    my $type = $self->type;
    return $name unless defined $type and $type ne 'folder';
    $name . $type;
}

#-------------------------------------------------------------------------------
sub AUTOLOAD {
    my $self = shift;
    my $class = ref( $self );
    unless ( $class ) {
	my $error = "$self is not an object";
	WWW::Yahoo::Briefcase::Exception::Usage->throw( $error );
    }
    my $name = $AUTOLOAD;
    { $name =~ s!.*:!! };
    unless ( $autoload->{$name} ) {
	my $error = "can't access '$name' field in class $class";
	WWW::Yahoo::Briefcase::Exception::Usage->throw( $error );
    }
    if ( @_ ) {
	my $error = "can't change read-only '$name' field in class $class";
	WWW::Yahoo::Briefcase::Exception::Usage->throw( $error );
    }
    $self->{$name};
}
sub DESTROY {};

################################################################################

=pod

=head1 name

  my $name = $object->name;

=head1 type

  my $type = $object->type;

=head1 fullname

  my $fullname = $object->fullname;
  my $fullname = "$object";

Full name includes the extension (also known as the type, unless the
object is a folder, in which case the type is 'folder').

=head1 size

  my $size = $object->size;

=head1 time

  my $time = $object->time;

=head1 id

  my $id = $object->id;

=cut

#-------------------------------------------------------------------------------

################################################################################

=pod

=head1 SEE ALSO

L<WWW::Yahoo::Briefcase|WWW::Yahoo::Briefcase>,
L<Exception::Class|Exception::Class>

=head1 AUTHOR

Ken Neighbors, Ph.D.
ken@nsds.com

=head1 COPYRIGHT AND LICENSE

Copyright 2003 by Ken Neighbors

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself, either Perl version 5.000 or (at
your option) any later version of Perl.  That is, you can redistribute
this module and/or modify it under the terms of either:

  a) the GNU General Public License as published by the Free Software
     Foundation; either version 1, or (at your option) any later version,
     or

  b) the "Artistic License" which comes with Perl, or

  c) the license terms of Perl versions later than 5.000

=cut

1;
